@@ -165,7 +165,7 @@ ews_send_to_sync (CamelTransport *transport,
 
 	success = camel_ews_utils_create_mime_message (
 		cnc, "SendOnly", NULL, message, 0,
-		from, NULL, NULL, cancellable, error);
+		from, recipients, NULL, NULL, cancellable, error);
 
 	g_object_unref (cnc);
 
